import { Logger } from "./Logger.js";
export function createBus() {
    const subscriptions = {};
    function subscribers(type) {
        let arr = subscriptions[type];
        if (!arr) {
            arr = [];
            subscriptions[type] = arr;
        }
        return arr;
    }
    return {
        publish(type, properties) {
            Logger.print("debug", `Publishing event`, type, properties);
            const payload = {
                type,
                properties
            };
            for (const sub of subscribers(type))
                sub.cb(payload);
        },
        unsubscribe(sub) {
            const arr = subscribers(sub.type);
            const index = arr.indexOf(sub);
            if (!index)
                return;
            arr.splice(index, 1);
        },
        subscribe(type, cb) {
            const sub = {
                type,
                cb
            };
            subscribers(type).push(sub);
            return sub;
        }
    };
}
